from typing import Optional, List

from db_hj3415.mymongo.base import Base
from utils_hj3415 import setup_logger
from dataclasses import dataclass, asdict

mylogger = setup_logger(__name__, level='INFO')

class Flask(Base):
    COL_TITLE = ('favorites',)

    def __init__(self, table: str):
        super().__init__(db='flask', table=table)

    @property
    def table(self) -> str:
        return self.table

    @table.setter
    def table(self, table: str):
        assert table in self.COL_TITLE, f'Invalid value : {table}({self.COL_TITLE})'
        self.table = table

@dataclass
class Stock:
    code: str
    name: str
    purchase_price: float
    purchase_quantity: int


class Favorites(Flask):
    def __init__(self):
        super().__init__('favorites')
        self.collection = self.mongo_client['flask']['favorites']

    def save(self, data:Stock) -> bool:
        result = self.collection.insert_one(asdict(data)) # type: ignore
        return result.acknowledged

    def get(self, code: str) -> Optional[Stock]:
        data = self.collection.find_one({"code": code}, {"_id": 0})  # _id 제외
        if data:
            stock_obj = Stock(**data)  # dict → dataclass 변환
            mylogger.info(f'Stock : {stock_obj}')
            return stock_obj

    def get_all(self) -> List[Stock]:
        documents = list(self.collection.find({}, {"_id": 0}))  # 모든 문서 조회, _id 제외
        return [Stock(**doc) for doc in documents]  # 변환

    def get_all_codes(self) -> list:
        return [stock.code for stock in self.get_all()]

